/****
HidTouch input driver.
(c)2008 David SPORN
 ****/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef HIDTOUCH__BODY__H
#define HIDTOUCH__BODY__H



/**** Device control event handler -- declaration ****/
static int 
hdtOnDeviceInit(DeviceIntPtr device) ;

static int 
hdtOnDeviceInit__initButtons(DeviceIntPtr device);

static int 
hdtOnDeviceInit__initAxes(DeviceIntPtr device);

static int 
hdtOnDeviceOn(DeviceIntPtr device) ;

static int 
hdtOnDeviceOff(DeviceIntPtr device) ;

static int 
hdtOnDeviceClose(DeviceIntPtr device) ;

static int 
hdtOnReadInput(InputInfoPtr pInfo) ;



/**** Device control event handler -- definition ****/
static int 
hdtOnDeviceInit(DeviceIntPtr device)
{
    hdtOnDeviceInit__initButtons(device) ;
    hdtOnDeviceInit__initAxes(device) ;

    return Success;
}

static int 
hdtOnDeviceOn(DeviceIntPtr device)
{
    InputInfoPtr pInfo = device->public.devicePrivate;
    HdtDevicePtr pDevice = pInfo->private;

    xf86Msg(X_INFO, "%s (%s): On.\n", pInfo->name, pDevice->myConfiguration.device);
    if (device->public.on)
    {
        xf86Msg(X_ERROR, "%s: device already open.\n", pInfo->name);
        return BadRequest;
    }
            
    SYSCALL(pInfo->fd = open(pDevice->myConfiguration.device, O_RDONLY | O_NONBLOCK));
    if (pInfo->fd < 0)
    {
        xf86Msg(X_ERROR, "%s: cannot open device.\n", pInfo->name);
        return BadRequest;
    } 

    xf86FlushInput(pInfo->fd);
    xf86AddEnabledDevice(pInfo);
    device->public.on = TRUE;

    return Success;
}

static int 
hdtOnDeviceOff(DeviceIntPtr device)
{
    InputInfoPtr pInfo = device->public.devicePrivate;
    HdtDevicePtr pDevice = pInfo->private;

    xf86Msg(X_INFO, "%s: Off.\n", pInfo->name);
    if (!device->public.on)
    {
        xf86Msg(X_ERROR, "%s: device already off.\n", pInfo->name);
        return BadRequest;
    }

    xf86RemoveEnabledDevice(pInfo);
    pInfo->fd = -1;
    device->public.on = FALSE;

    return Success;
}

static int 
hdtOnDeviceClose(DeviceIntPtr device)
{
    return Success;
}


static int 
hdtOnDeviceInit__initButtons(DeviceIntPtr device)
{
    InputInfoPtr        pInfo = device->public.devicePrivate;
    CARD8               *map;
    int                 i;
    const int           num_buttons = 3;
    int                 ret = Success;

    map = xcalloc(num_buttons, sizeof(CARD8));

    for (i = 0; i < num_buttons; i++)
    {
        map[i] = i;
    }

    if (!InitButtonClassDeviceStruct(device, num_buttons, map)) {
        xf86Msg(X_ERROR, "%s: Failed to register buttons.\n", pInfo->name);
        ret = BadAlloc;
    }

    xfree(map);
    return ret;
}

static int 
hdtOnDeviceInit__initAxes(DeviceIntPtr device)
{
    InputInfoPtr pInfo = device->public.devicePrivate;
    HdtDevicePtr pDevice = pInfo->private;
    const int           num_axes = 2;


    if (!InitValuatorClassDeviceStruct(device,
            num_axes, GetMotionHistory, GetMotionHistorySize(),
            Absolute))
    {
        return BadAlloc;
    }

    pInfo->dev->valuator->mode = Absolute;
    if (!InitAbsoluteClassDeviceStruct(device))
    {
        return BadAlloc;
    }
    
    ScrnInfoPtr   pScreen = xf86Screens[0];

    xf86InitValuatorAxisStruct(device, 0,
            pDevice->myConfiguration.calibration.minX,
            pDevice->myConfiguration.calibration.maxX, 
            pScreen->virtualX,
            0,
            pScreen->virtualX);
    xf86InitValuatorDefaults(device, 0);

    xf86InitValuatorAxisStruct(device, 1,
            pDevice->myConfiguration.calibration.minY,
            pDevice->myConfiguration.calibration.maxY, 
            pScreen->virtualY,
            0,
            pScreen->virtualY);
    xf86InitValuatorDefaults(device, 1);


    return Success;
}

static int 
hdtOnReadInput(InputInfoPtr pInfo)
{
    HdtDevicePtr pDevice = pInfo->private;

    HdtRawDataRec raw_data ;
    raw_data.pressure = -1 ;
    raw_data.x = -1 ;
    raw_data.y = -1 ;
    HdtRawData__fillFromInputInfo(&raw_data, pInfo) ;

    int has_changed = 0 ;
    if ((-1 < raw_data.x && raw_data.x != pDevice->myRawData.x) 
            || (-1 < raw_data.y && raw_data.y != pDevice->myRawData.y))
    {
        has_changed = 1 ;
        pDevice->myRawData.x = raw_data.x ;
        pDevice->myRawData.y = raw_data.y ;

        xf86PostMotionEvent(pInfo->dev, 1
                , 0, 2
                , raw_data.x, raw_data.y);
    } 
    if (-1 < raw_data.pressure && raw_data.pressure != pDevice->myRawData.pressure)
    {
        has_changed = 1 ;
        pDevice->myRawData.pressure = raw_data.pressure ;
        xf86PostButtonEvent(pInfo->dev, 1, HDT__BUTTON_EVENT__LEFT_BUTTON
                ,pDevice->myRawData.pressure ,0, 2
                ,raw_data.x, raw_data.y) ;
    } 

    if (1 == has_changed)
    {
        pDevice->myRawData.timestamp = GetTimeInMillis();
    }
    
    return Success ;
}


#endif /** ifndef HIDTOUCH__BODY__H **/
